#
# Copyright (C) 2021 Associated Universities, Inc. Washington DC, USA.
#
# This file is part of NRAO Workspaces.
#
# Workspaces is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Workspaces is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Workspaces.  If not, see <https://www.gnu.org/licenses/>.
""" Tests for CASA Matrix Service """

import pytest
from pycapo import CapoConfig
from sqlalchemy import create_engine
from sqlalchemy.orm import Session

from workspaces.capability.schema import CasaMatrixCasaVersion
from workspaces.capability.schema import mapper_registry as capability_mapper_registry
from workspaces.system.services.casa_matrix_service import CasaMatrixService

settings = CapoConfig().settings("edu.nrao.archive.workflow.config.CasaVersions")

DEFAULT_CASA = settings["homeForDownloads"]
VLA_DEFAULT_CASA = settings["homeForVlaRestore"]
ALMA_DEFAULT_CASA = settings["homeForAlmaRestore"]
VUDI_DEFAULT_CASA = settings["homeForVlaUserDrivenImaging"]
AUDI_DEFAULT_CASA = settings["homeForAlmaUserDrivenImaging"]


@pytest.fixture(scope="module")
def fake_version() -> CasaMatrixCasaVersion:
    return CasaMatrixCasaVersion.from_json(CasaMatrixCasaVersion, {
            "casa_version": "1.2.3-4|default",
            "capabilities": ["restore_cms", "std_calibration"],
            "is_cluster_compatible": True,
        }
    )


@pytest.fixture(scope="module")
def mock_matrix_service() -> CasaMatrixService:
    """
    Pytest fixture providing a mock CasaMatrixService object; mocked object can be used
    like a normal CasaMatrixService instance.

    :return: Mocked CasaMatrixService
    """

    engine = create_engine("sqlite:///:memory:")
    capability_mapper_registry.metadata.create_all(engine)
    session = Session(engine)

    mocked_matrix_service = CasaMatrixService(session)

    yield mocked_matrix_service


class TestCasaMatrixService:
    """Test class for CasaMatrixService"""

    @pytest.mark.skip(reason="Directory structure in the container isn't properly set up")
    def test_make_links(self, mock_matrix_service: CasaMatrixService):
        ret_links = mock_matrix_service.make_links()
        assert len(ret_links) > 0

    @pytest.mark.skip(reason="Default lookup is getting KeyErrors")
    @pytest.mark.parametrize("fake_request", [
        {"casa_version": None, "capability": None, "telescope": None},
        {"casa_version": None, "capability": "std_calibration", "telescope": None}
    ])
    def test_default_version(self, mock_matrix_service: CasaMatrixService, fake_request: dict[str, str, str]):
        expected_version = DEFAULT_CASA
        ret_version = mock_matrix_service.get_version(None, fake_request["capability"])
        assert ret_version["version"] == expected_version

    @pytest.mark.skip(reason="Default lookup is getting KeyErrors")
    @pytest.mark.parametrize("fake_request", [
        {"casa_version": "5.6.1-8|default", "capability": "restore_cms", "telescope": "VLA"},
        {"casa_version": "5.6.1-8|default", "capability": "restore_basic_cms", "telescope": "VLA"}
    ])
    def test_vla_default_version(self, mock_matrix_service: CasaMatrixService, fake_request: dict[str, str, str]):
        expected_version = VLA_DEFAULT_CASA
        ret_version = mock_matrix_service.get_version(None, fake_request["capability"], fake_request["telescope"])
        assert ret_version["version"] == expected_version

    @pytest.mark.skip(reason="Default lookup is getting KeyErrors")
    @pytest.mark.parametrize("fake_request", [
        {"casa_version": "5.6.1-8|default", "capability": "restore_cms", "telescope": "ALMA"},
        {"casa_version": "5.6.1-8|default", "capability": "restore_basic_cms", "telescope": "ALMA"}
    ])
    def test_alma_default_version(self, mock_matrix_service: CasaMatrixService, fake_request: dict[str, str, str]):
        expected_version = ALMA_DEFAULT_CASA
        ret_version = mock_matrix_service.get_version(None, fake_request["capability"], fake_request["telescope"])
        assert ret_version["version"] == expected_version

    @pytest.mark.skip(reason="Default lookup is getting KeyErrors")
    @pytest.mark.parametrize("fake_request", [
        {"casa_version": "5.6.1-8|default", "capability": None, "telescope": None},
        {"casa_version": "6.1.1-15|2020.1.0.40", "capability": None, "telescope": None}
    ])
    def test_valid_version(self, mock_matrix_service: CasaMatrixService, fake_request: dict[str, str, str]):
        expected_version = fake_request["casa_version"]
        ret_version = mock_matrix_service.get_version(fake_request["casa_version"])
        assert ret_version["version"] == expected_version

    @pytest.mark.skip(reason="Default lookup is getting KeyErrors")
    @pytest.mark.parametrize("fake_request", [
        {"casa_version": "0.0.0-0|default", "capability": None, "telescope": None},
        {"casa_version": "x.x.x-x|default", "capability": None, "telescope": None}
    ])
    def test_invalid_version(self, mock_matrix_service: CasaMatrixService, fake_request: dict[str, str, str]):
        expected_version = DEFAULT_CASA
        ret_version = mock_matrix_service.get_version(fake_request["casa_version"])
        assert ret_version["version"] == expected_version

    @pytest.mark.skip(reason="Default lookup is getting KeyErrors")
    @pytest.mark.parametrize("fake_request", [
        {"casa_version": "0.0.0-0|default", "capability": "restore_cms", "telescope": "VLA"},
        {"casa_version": "x.x.x-x|default", "capability": "restore_basic_cms", "telescope": "VLA"}
    ])
    def test_vla_invalid_version(self, mock_matrix_service: CasaMatrixService, fake_request: dict[str, str, str]):
        expected_version = VLA_DEFAULT_CASA
        ret_version = mock_matrix_service.get_version(
            fake_request["casa_version"], fake_request["capability"], fake_request["telescope"]
        )
        assert ret_version["version"] == expected_version

    @pytest.mark.skip(reason="Default lookup is getting KeyErrors")
    @pytest.mark.parametrize("fake_request", [
        {"casa_version": "0.0.0-0|default", "capability": "restore_cms", "telescope": "ALMA"},
        {"casa_version": "x.x.x-x|default", "capability": "restore_basic_cms", "telescope": "ALMA"}
    ])
    def test_alma_invalid_version(self, mock_matrix_service: CasaMatrixService, fake_request: dict[str, str, str]):
        expected_version = ALMA_DEFAULT_CASA
        ret_version = mock_matrix_service.get_version(
            fake_request["casa_version"], fake_request["capability"], fake_request["telescope"]
        )
        assert ret_version["version"] == expected_version

    @pytest.mark.skip(reason="DB lookup isn't returning any results")
    @pytest.mark.parametrize("fake_capability", ["std_calibration"])
    def test_std_calibration_recipe(self, mock_matrix_service: CasaMatrixService, fake_capability: str):
        expected_recipe = "procedure_hifv.xml"
        ret_recipe = mock_matrix_service.get_recipe(fake_capability)
        assert ret_recipe == expected_recipe

    @pytest.mark.skip(reason="DB lookup isn't returning any results")
    @pytest.mark.parametrize("fake_capability", ["restore_cms", "restore_basic_ms"])
    def test_restore_recipe(self, mock_matrix_service: CasaMatrixService, fake_capability: str):
        expected_recipe = "template_hifv_deliver_ms.xml|template_hifa_deliver_ms.xml"
        ret_recipe = mock_matrix_service.get_recipe(fake_capability)
        assert ret_recipe == expected_recipe

    @pytest.mark.skip(reason="DB lookup isn't returning any results")
    @pytest.mark.parametrize("fake_capability", ["std_cms_imaging", "std_restore_imaging"])
    def test_restore_recipe(self, mock_matrix_service: CasaMatrixService, fake_capability: str):
        expected_recipe = "procedure_hifv_contimage_selfcal.xml"
        ret_recipe = mock_matrix_service.get_recipe(fake_capability)
        assert ret_recipe == expected_recipe

    @pytest.mark.skip(reason="DB lookup isn't returning any results")
    def test_get_versions(self, mock_matrix_service: CasaMatrixService):
        ret_versions = mock_matrix_service.get_versions()
        assert len(ret_versions) > 0

    @pytest.mark.skip(reason="DB lookup isn't returning any results")
    @pytest.mark.parametrize("fake_request", [
        {"casa_version": "5.6.1-8|default", "capability": None, "telescope": None},
        {"casa_version": "6.1.1-15|2020.1.0.40", "capability": None, "telescope": None}
    ])
    def test_get_versions_recommended(self, mock_matrix_service: CasaMatrixService, fake_request: dict[str, str, str]):
        expected_recommended_version = fake_request["casa_version"]
        ret_versions = mock_matrix_service.get_versions(fake_request["casa_version"])
        assert len(ret_versions) > 0
        assert ret_versions[0]["version"] == expected_recommended_version

    def test_add_version(self, mock_matrix_service: CasaMatrixService, fake_version: CasaMatrixCasaVersion):
        ret = mock_matrix_service.add_version(
            fake_version.casa_version, fake_version.capabilities, fake_version.is_cluster_compatible
        )
        mock_matrix_service.delete_version(fake_version.casa_version)
        assert ret

    def test_update_version(self, mock_matrix_service: CasaMatrixService, fake_version: CasaMatrixCasaVersion):
        mock_matrix_service.add_version(fake_version.casa_version)
        ret = mock_matrix_service.update_version(
            fake_version.casa_version, fake_version.capabilities, fake_version.is_cluster_compatible
        )
        mock_matrix_service.delete_version(fake_version.casa_version)
        assert ret

    def test_delete_version(self, mock_matrix_service: CasaMatrixService, fake_version: CasaMatrixCasaVersion):
        mock_matrix_service.add_version(fake_version.casa_version)
        ret = mock_matrix_service.delete_version(fake_version.casa_version)
        assert ret
